import cvxpy as cp
import numpy as np


def sdp(rho0, rho1):
    n = 2
    t = 0.5  # Since t is arbitrary, change to whatever you like.
    C = np.zeros((4, 4))
    C[0:2, 0:2] = (1 - t) * rho1.T
    C[2:4, 2:4] = t * rho0.T
    A = []
    b = []

    A0 = np.zeros((4, 4))
    A0[0:2, 0:2] = rho1.T
    A0[2:4, 2:4] = -1 * (rho0.T)
    A.append(A0)
    b.append(0)
    for i in range(n):
        for j in range(n):
            E = np.zeros((4, 4))
            E[i][j] = 1
            E[n + i][n + j] = 1
            A.append(E.T)
            b.append(1 if i == j else 0)

    # Define and solve the CVXPY problem.
    # Create a symmetric matrix variable.
    X = cp.Variable((4, 4), symmetric=True)
    # The operator >> denotes matrix inequality.
    constraints = [X >> 0]
    constraints += [
        cp.trace(A[i] @ X) == b[i] for i in range(n**2 + 1)
    ]
    prob = cp.Problem(cp.Minimize(cp.trace(C @ X)),
                      constraints)
    prob.solve()

    # DEBUGGING.
    # print(A)
    # print(b)

    # Print result.
    print("Input rho_0 is:")
    print(rho0)
    print("Input rho_1 is:")
    print(rho1)
    print("The optimal value (min_pwc) is", prob.value)
    print("A solution X (composed of P0 and P1) is")
    '''
    Change desired decimal places for solution here
    '''
    solution = np.around(X.value, decimals=3)
    print(solution)
    print("P0 is:")
    print(solution[0:2, 0:2])
    print("P1 is:")
    print(solution[2:4, 2:4])


if __name__ == "__main__":
    '''
    Change rho_0 and rho_1 here
    '''
    rho0 = np.array([[1, 0], [0, 0]])
    rho1 = np.array([[0.5, 0], [0, 0.5]])
    sdp(rho0, rho1)



import pathlib
from setuptools import setup

# The directory containing this file
HERE = pathlib.Path(__file__).parent

# The text of the README file
README = (HERE / "README.md").read_text()

setup(
    name='helstrom',
    packages=['helstrom'],
    version='0.1',
    license='MIT',
    description=' A quantum state distinguisher with minimum theoretical worst-case probability of error.',
    long_description=README,
    long_description_content_type="text/markdown",
    author='Benedictus Alvian Sofjan',
    author_email='benedictus.sofjan@gmail.com',
    url='https://github.com/benedictusalvian/helstrom',
    download_url='https://github.com/benedictusalvian/helstrom/archive/refs/tags/0.1.tar.gz',
    keywords=['python', 'quantum', 'quantum-computing',
              'quantum-information', 'quantum-theory', 'helstrom', 'distinguisher'],
    install_requires=[
        'cvxpy == 1.1.12',
        'Mosek == 9.2.43',
        'numpy == 1.20.3'
    ],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.8',
    ],
    include_package_data=True,
)
